@@ -1,4 +1,36 @@
+from __future__ import with_statement
+
 import os, sys, types, cPickle, stat, re, itertools, time, hashlib, math
+if sys.version_info >= (2,6):
+    from functools import reduce
+
+if sys.version_info >= (3,2):
+    from functools import cmp_to_key
+elif sys.version_info >= (3,):
+    def cmp_to_key(mycmp):
+        """Convert a cmp= function into a key= function.  Copied from
+        http://wiki.python.org/moin/HowTo/Sorting/, where it states
+        this is the fucntion that has been incorporated into the
+        functools module in versions 3.2 and 3.3 of Python.
+
+        """
+        class K(object):
+            def __init__(self, obj, *args):
+                self.obj = obj
+            def __lt__(self, other):
+                return mycmp(self.obj, other.obj) < 0
+            def __gt__(self, other):
+                return mycmp(self.obj, other.obj) > 0
+            def __eq__(self, other):
+                return mycmp(self.obj, other.obj) == 0
+            def __le__(self, other):
+                return mycmp(self.obj, other.obj) <= 0
+            def __ge__(self, other):
+                return mycmp(self.obj, other.obj) >= 0
+            def __ne__(self, other):
+                return mycmp(self.obj, other.obj) != 0
+        return K    
+
 
 ##################################################
 ## Syntax extensions
@@ -75,29 +107,51 @@ class SnooperMixin(object):
     #     self.snoop = set()
     #     super(SnooperMixin, self).__init__(*a, **kw)
 
-    def __getattribute__(self, name):
-        def safe_getattr(the_name):
-            return super(SnooperMixin, self).__getattribute__(the_name)
-        # Create an place to store results if necessary.  Could do
-        # this with the modified __init__ commented above, but want to
-        # isolate my changes to a single function.
-        try:
-            snoop = safe_getattr('snoop')
-        except AttributeError:
-            snoop = set()
-            self.snoop = snoop
-        # Do whatever you want here, making sure not to do anything
-        # that explicitly or implicitly calls self.__getattribute__().
-        # In this case just build up a list of accessed attributes.
-        if name is not 'snoop': 
-            snoop.add(name)
-        # Return the actual requested attribute here.
-        return safe_getattr(name)
+    if sys.version_info < (3,):
+        # version for Python 2
+        def __getattribute__(self, name):
+            def safe_getattr(the_name):
+                return super(SnooperMixin, self).__getattribute__(the_name)
+            # Create an place to store results if necessary.  Could do
+            # this with the modified __init__ commented above, but want to
+            # isolate my changes to a single function.
+            try:
+                snoop = safe_getattr('snoop')
+            except AttributeError:
+                snoop = set()
+                self.snoop = snoop
+            # Do whatever you want here, making sure not to do anything
+            # that explicitly or implicitly calls self.__getattribute__().
+            # In this case just build up a list of accessed attributes.
+            if name is not 'snoop': 
+                snoop.add(name)
+            # Return the actual requested attribute here.
+            return safe_getattr(name)
+    else:        
+        # version for Python 3
+        def __getattribute__(self, name):
+            # Create an place to store results if necessary.  Could do
+            # this with the modified __init__ commented above, but want to
+            # isolate my changes to a single function.
+            try:
+                snoop = super().__getattribute__('snoop')
+            except AttributeError:
+                snoop = set()
+                self.snoop = snoop
+            # Do whatever you want here, making sure not to do anything
+            # that explicitly or implicitly calls self.__getattribute__().
+            # In this case just build up a list of accessed attributes.
+            if name is not 'snoop': 
+                snoop.add(name)
+            # Return the actual requested attribute here.
+            return super().__getattribute__(name)
 
 class Container (object):
     """Simple object that has a nice syntax for filling in a few
-    attributes:
-    >>> c = Container(a=1, b=2)"""
+    attributes: 
+    >>> c = Container(a=1, b=2)
+
+    """
     def __init__(self, **kw):
         for k in kw: self.__setattr__(k, kw[k])
 
@@ -125,6 +179,18 @@ class DotDict(dict):
     methods, this causes no problem.
 
     """
+    # Note that Peter Norvig's answer to almost the same question in
+    # Python Infrequently Asked Questions is this:
+    # 
+    # class Struct:
+    #     def __init__(self, **entries): self.__dict__.update(entries)
+    # 
+    # That definitely allows nicer creation Struct(a=1, b=2), but
+    # that's not my only requirement here.  I also want .keys(), etc.  So I
+    # guess I'll stick with my implementation.
+    # 
+    # But then, I didn't know about the vars() builtin function, which
+    # perhaps gives me what I want: vars(obj) -> dict of attributes.
     def __getattribute__(self, name):
         # If there's an attribute of this name, return it
         try: return dict.__getattribute__(self, name)
@@ -366,7 +432,20 @@ def forkify(f):
 ##################################################
 ## Generic convenient Functions
 ##################################################
-    
+
+def isstring(obj):
+    """Test if an object is a string for different python versions."""
+    # Early Python only had one string type
+    # if type(obj) is str
+    # Middle-aged Python had several:
+    # if type(obj) in types.StringTypes
+    # Modern python has one again
+    # if type(obj) is str
+    if sys.version_info < (3,):
+        return type(obj) in types.StringTypes
+    else:
+        return type(obj) is str
+
 def wrap(x, min, max):
     """Wrap x periodically into the range [min, max)"""
     return ((x - min) % (max - min)) + min
@@ -407,7 +486,7 @@ def prime_factorize(n):
     elif pfs[0] == n:
         return pfs
     else:
-        return [pfs[0]] + prime_factorize(n/pfs[0])
+        return [pfs[0]] + prime_factorize(n//pfs[0])
 
 ##################################################
 ## Functions for manipulating lists
@@ -519,10 +598,11 @@ def same(x,y):
     This is the same as if you just test with the equality operator,
     except that same() treats lists and tuples as the same
 
-    >>> [1, [2, 3]] == [1, (2, 3)]]
+    >>> [1, [2, 3]] == [1, (2, 3)]
     False
-    >>> same([1, [2, 3]] == [1, (2, 3)]])
+    >>> same([1, [2, 3]], [1, (2, 3)])
     True
+
     """
     if iterable(x):
         if len(x) != len(y): return False
@@ -531,13 +611,27 @@ def same(x,y):
 
 def merge_comp(l1, l2):
     """Make a comparison function for merge_sorted"""
+    # cmp() is gone in Python3.  But we're using the cmp_to_key
+    # wrapper, so make a fucntion that behaves exactly like cmp()...
+    # The 'right' thing to do would be to reformulate this so we just
+    # use sorted( key=...), and therefore don't need cmp() or
+    # cmp_to_key
+    if sys.version_info >= (3,):
+        def local_cmp(x,y):
+            if x < y: return -1
+            elif x == y: return 0
+            elif x > y: return 1
+            else: raise RuntimeError # should be impossible
+    else:
+        local_cmp = cmp
+
     def comp(x,y):
         if   x in l1 and y in l1:
-            return cmp(l1.index(x), l1.index(y))
+            return local_cmp(l1.index(x), l1.index(y))
         elif x in l2 and y in l2:
-            return cmp(l2.index(x), l2.index(y))
-        elif x not in l1 and x not in l2 or \
-             y not in l1 and y not in l2:
+            return local_cmp(l2.index(x), l2.index(y))
+        elif (x not in l1 and x not in l2) or \
+             (y not in l1 and y not in l2):
             # This should never happen
             raise RuntimeError, "This should never happen"
         else:
@@ -583,7 +677,12 @@ def merge_sorted(l1, l2, check=True):
                 assert cmp1(e1, e2) ==  cmp2(e1, e2)
                 assert cmp1(e1, e2) == -cmp1(e2, e1)
                 assert cmp2(e1, e2) == -cmp2(e2, e1)
-    return sorted(all, cmp=merge_comp(l1, l2))
+
+    if sys.version_info > (3,):
+        result = sorted(all, key=cmp_to_key(merge_comp(l1, l2)))
+    else:
+        result = sorted(all, cmp=merge_comp(l1, l2))
+    return result 
 
 def grab_list(lst, pred):
     """Return a list of elements of lst that satisfy predicate.  The
@@ -919,7 +1018,7 @@ def timer(f, *a, **kw):
         reps *= factor
 
     # reps was multipled one too many times
-    reps /= factor
+    reps = reps // factor
     return dt / reps, reps
 
 ##################################################
