@@ -1,3 +1,5 @@
+from __future__ import with_statement
+
 import unittest, tempfile
 
 try: import numpy
@@ -67,7 +69,11 @@ class SyntaxTest(unittest.TestCase):
         self.assertEqual(obj.b, 2)
                     
     def test_BracketAccess(self):
-        class testNewObj (object, BracketAccessMixin): pass
+        # Note method resolution order fails if you do it this way b/c
+        # MRO hits all the methods of object first, before those of
+        # BracketAccessMixin (which also inherits from object) class
+        # testNewObj (object, BracketAccessMixin): pass
+        class testNewObj (BracketAccessMixin, object): pass
         class testOldObj (BracketAccessMixin): pass
         self.bracketAccessTest(testNewObj())
         self.bracketAccessTest(testOldObj())
@@ -363,7 +369,8 @@ class FileTest(unittest.TestCase):
         
     def test_import_graph(self):
         # This just checks for exceptions, not correct behavior
-        with tempfile.TemporaryFile() as outf:
+        # Python 3 is picky about binary vs. text mode file output
+        with tempfile.TemporaryFile(mode='w') as outf:
             import_graph(out_file=outf)
             import_graph(out_file=outf, with_system=False)
             import_graph(out_file=outf, excludes=['sys'])
@@ -379,10 +386,12 @@ class FileTest(unittest.TestCase):
         os.remove(tfn)
 
         # file object
-        f = open(tfn, 'w')
+        # In Python 3, the file must be opened in binary mode
+        f = open(tfn, 'wb')
         can([1,2,3], f)
         f.close()
-        f = open(tfn)
+        # In Python 3, the file must be opened in binary mode
+        f = open(tfn, 'rb')
         self.assertEqual(uncan(f), [1,2,3])
         f.close()
         os.remove(tfn)
@@ -393,7 +402,9 @@ class FileTest(unittest.TestCase):
         os.remove(tfn)
         
     def test_file_size(self):
-        tf = tempfile.NamedTemporaryFile()
+        # Python 3 is pickier about text vs. binary: require opening
+        # in text mode to write a string to the file.
+        tf = tempfile.NamedTemporaryFile(mode='w')
         tf.file.write('abcd')
         tf.file.flush()
         self.assertEqual(file_size(tf.name), 4)
